﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RegularPolygons
{
    public class Polygon
    {
        public float Radius { get; private set; }//radio
        public Point Center { get; private set; }//punto centro
        public int Sides { get; private set; }//lados del polígono
        public Point[] Vertexs { get; private set; }//vértices del polígono
        public float Angle { get; private set; }//ángulo de rotación del polígono

        public Polygon(Point center, int sides)
        {
            this.Center = center;
            this.Sides = sides;
        }

        public void Update(Point endPoint)
        {
            //obtiene el radio partiendo desde el centro
            Radius = (float)Math.Sqrt(Math.Pow(Center.X - endPoint.X, 2) + Math.Pow(Center.Y - endPoint.Y, 2));

            //obtiene el ángulo de una línea imaginaria(radio) entre el centro y el vértice
            PointToDegree(Center, endPoint);

            //obtiene los vértices del polígono
            GetVertexs(Sides, Radius, Angle, Center);
        }


        //obtiene los vértices del polígono, teniendo en cuenta el radio, los lados y el ángulo de rotación
        private void GetVertexs(int sides, float radius, float angle, Point startPoint)
        {
            //por supuesto para que sea un polígono debe haber as de 2 lados
            if (sides > 2)
            {
                List<Point> points = new List<Point>();
                float vCount = 360.0f / sides; //la cantidad de vértices a buscar es la longitud de la circunferencia entre la cantidad de lados del polígono

                float _angle = angle; //_angle Almacena el ángulo de la supuesta línea que habrá entre el punto central y un vértice
                                      //al empezar comienza con el mismo ángulo de inicio

                //Recorre la circunferencia ángulo por ángulo según la cantidad de vértices, comenzando por el propio ángulo inicial
                for (float i = angle; i < angle + 360.0; i += vCount) 
                {
                    //obtiene el punto en la frontera de la circunferencia relacionado con el ángulo y el vértice y lo agrega a la lista de vértices
                    points.Add(DegreesToPoint(_angle, radius, startPoint));

                    //pasa al siguiente ángulo
                    _angle += vCount;
                }

                Vertexs = points.ToArray();
            }
        }

        //obtiene el ángulo de una línea imaginaria entre el centro y el vértice
        //en este ejemplo la línea imaginaria es la línea que debería haber entre el centro y la posición actual del cursor, o sea el radio
        private void PointToDegree(Point startPoint, Point endPoint)
        {
            int deltaX = startPoint.X - endPoint.X;
            int deltaY = startPoint.Y - endPoint.Y;

            double radAngle = Math.Atan2(deltaY, deltaX);
            double degreeAngle = radAngle * 180.0 / Math.PI;

            Angle = (float)(180.0 - degreeAngle);
        }

        //Obtiene el punto extremo de una línea imaginaria entre el centro y el vértice, teniendo en cuenta al ángulo de esta línea
        //en este ejemplo la línea imaginaria es la línea que debería haber entre el centro y la posición actual del cursor, o sea el radio
        private Point DegreesToPoint(float degrees, float radius, Point startPoint)
        {
            Point point = new Point();
            double radians = degrees * Math.PI / 180.0;

            point.X = (int)(Math.Cos(radians) * radius + startPoint.X);
            point.Y = (int)(Math.Sin(-radians) * radius + startPoint.Y);

            return point;
        }

    }
}
